from typing import TypedDict
import logging

HAS_RICH = True

try:
    from rich.logging import RichHandler
    from rich.highlighter import Highlighter
    from rich.text import Text
except ImportError:
    HAS_RICH = False

class DriverOptionsStdout(TypedDict):
    ...

def driver_stdout(driver_option: DriverOptionsStdout = None):
    if not HAS_RICH:
        raise ImportError(
        "You must install rich to use the stdout driver. "
        "Run: pip install rich"
    )

    class LogNameHighlighter(Highlighter):
        def highlight(self, text: Text) -> None:
            text.highlight_regex(r"^\w+ - ", style="black italic")

    rich_handler = RichHandler(
        highlighter=LogNameHighlighter(),
        level=logging.DEBUG,
        omit_repeated_times=False,
        tracebacks_show_locals=True,
        tracebacks_extra_lines=True,
        tracebacks_word_wrap=True,
        rich_tracebacks=True,
    )

    formatter_string = "%(name)s - %(message)s"

    formatter = logging.Formatter(formatter_string)
    rich_handler.setFormatter(formatter)

    return rich_handler



import re

from EscribaLogger.process_extra_context import process_extra_context


def test_should_process_extra_context():
    sample_dict = {"key": "value"}
    result = process_extra_context(sample_dict)
    assert result == ' - {"key": "value"}'


def test_should_return_empty_string_when_empty_context():
    result = process_extra_context({})
    assert result == ""


def test_should_return_empty_string_when_context_is_none():
    result = process_extra_context(None)
    assert result == ""


def test_should_convert_object_instances_to_string():
    class Item:
        param = "Escriba param"

        def __init__(self) -> None:
            self.name = "the Escriba"

        def __str__(self):
            return "the Escriba"

    sample_dict = {"full_class": Item, "object": Item()}
    result = process_extra_context(sample_dict)

    assert type(result) is str
    assert re.search(r"\.Item", result)
    assert re.search(r"<object \(Item\)\>: the Escriba", result)
    assert re.search(r"\.Item", result)
    assert re.search(r"<object \(Item\)\>: the Escriba", result)



from typing import TypedDict, Optional, Literal
import logging

HAS_GRAYPY = True

try:
    import graypy
except ImportError:
    HAS_GRAYPY = False

class DriverOptionsGraylog(TypedDict):
    graylog_host: Optional[str]
    graylog_port: Optional[int]
    graylog_protocol: Optional[Literal["http", "udp"]]

def driver_graylog(driver_options: DriverOptionsGraylog = None):
    if not HAS_GRAYPY:
        raise ImportError(
            "You must install graypy to use the graylog driver. "
            "Run: pip install graypy"
        )

    graylog_host = driver_options.get("graylog_host", "localhost")
    graylog_port = driver_options.get("graylog_port", 12201)
    protocol = driver_options.get("graylog_protocol", "http")

    formatter_string = "%(name)s.%(levelname)s - %(message)s"
    formatter = logging.Formatter(formatter_string)
    stream = graypy.GELFHTTPHandler(graylog_host, graylog_port)

    if protocol == "udp":
        stream = graypy.GELFUDPHandler(graylog_host, graylog_port)
    elif protocol == "http":
        stream = graypy.GELFHTTPHandler(graylog_host, graylog_port)

    stream.setFormatter(formatter)
    return stream


from typing import TypedDict, Optional
import logging
import datetime
import os


class DriverOptionsFile(TypedDict):
    file_location: Optional[str]
    file_name: Optional[str]

def driver_file(driver_option: DriverOptionsFile = None):
    if not driver_option:
        driver_option = {"file_location": "logs"}

    formatter_string = "[%(asctime)s] "
    formatter_string += "%(name)s.%(levelname)s "
    formatter_string += "- %(message)s "
    formatter_string += "(%(filename)s:%(lineno)d) "
    formatter_string += "%(extra_context)s"

    formatter = logging.Formatter(formatter_string)
    # formatter.default_time_format = '%Y-%m-%d %H:%M:%s'

    now_time = datetime.datetime.now().strftime("%Y-%m-%d")
    default_filename = f"{now_time}.log"

    log_file_name = driver_option.get("file_name", default_filename)
    log_file_location = driver_option.get("file_location", "logs")

    log_file_path = os.path.join(log_file_location, log_file_name)

    stream = logging.FileHandler(log_file_path)
    stream.setFormatter(formatter)
    return stream



from typing import TypedDict, Optional, Literal, TypeAlias

from .driver_file import driver_file, DriverOptionsFile
from .driver_graylog import driver_graylog, DriverOptionsGraylog
from .driver_stdout import driver_stdout, DriverOptionsStdout

class DriverOptions(TypedDict):
    file: Optional[DriverOptionsFile]
    graylog: Optional[DriverOptionsGraylog]
    stdout: Optional[DriverOptionsStdout]

t_available_drivers: TypeAlias = Literal["file", "stdout", "graylog"]

__all__ = [
    "driver_file",
    "driver_graylog",
    "driver_stdout",

    "DriverOptions",
    "t_available_drivers",
]



import json


def process_extra_context(context: dict):
    if not context:
        return ""
    if type(context) is str:
        raise TypeError(
            f"Context must be a object dict, not {type(context).__name__}!!!"
        )

    for key, value in context.items():
        if isinstance(value, (str, int, float, bool, list, dict, tuple, set)):
            continue
        elif hasattr(value, "__str__"):
            class_name = value.__class__.__name__
            new_value = f"<object ({class_name})>: {str(value)}"
            context.update({key: new_value})

    return " - " + json.dumps(context)
